#include "bsp_ml307r.h"
#include "at_chat.h"
#include "at_port.h"
#include "bsp_uart.h"
#include "log.h"
#include "stdio.h"
#include "CONFIG.h"
#include "SLEEP.h"
#include "bsp_iwdg.h"
#include "bsp_bmp390.h"
#include "bsp_tim.h"
#include "time.h"
#include <stdlib.h>
#include "bsp_adc.h"
#include "lwjson.h"

#undef  LOG_ENABLE
#define LOG_ENABLE  1

IotFlagStruct  IotFlag_t;

// λڴ洢IoT
lwrb_t iot_send_queue;
uint8_t iot_send_queue_buffer[sizeof(TsIotSendRequest) * 5] = {0};

#define ML307_PERIODIC_EVT (0x0001 << 0)


MultiTimer Multi_Power_ON_t;  //
MultiTimer Multi_Power_OFF_t; //ػ
MultiTimer Multi_Restart_t;   //
MultiTimer Multi_Cpin_t;      //cpin
MultiTimer Multi_Connect_t;   //Connect
MultiTimer Multi_Send_t;      //Send
MultiTimer Multi_StopTask_t;  //StopTask
MultiTimer Multi_Loop_t;      //Loop
MultiTimer Multi_GetInfo_t;   //siminfo

static tmosTaskID ml307_task_id = INVALID_TASK_ID;

at_obj_t *ml307r_obj;


int Ml307r_Gpio_Init(void)
{
    GPIOB_SetBits(ENABLE_3_8_V);
    GPIOB_ModeCfg(ENABLE_3_8_V, GPIO_ModeOut_PP_5mA);

    GPIOB_ResetBits(ML307_RST_PIN);
    GPIOB_ModeCfg(ML307_RST_PIN, GPIO_ModeOut_PP_5mA);

    GPIOB_ModeCfg(USIM_DECT_PIN, GPIO_ModeIN_Floating);

    GPIOB_ModeCfg(ML307_PWR_PIN, GPIO_ModeOut_PP_5mA);

    return 0;
}

void At_Debug(const char *fmt, ...)
{
    va_list args;
    va_start(args, fmt);
    vprintf(fmt, args);
    va_end(args);
}

void Ml307_Delay_Ms(uint16_t ms)
{
    DelayMs(ms);
}

void Ml307_Delay_us(uint16_t us)
{
    DelayUs(us);
}

static const at_adapter_t at_adapter =
{
    .write  = BSP_Uart1_Send_Data,
    .read   = BSP_Uart1_Receive_Data,
    .debug  = At_Debug,
    .recv_bufsize   = 256,
    .urc_bufsize    = 512
};

static IoT_State_t state = IOT_STATE_POWER_ON; 
/*
 * ʱ,λ:ms,Ĭ:10s
 */
#define   POWERON_RETRY_INTERVAL_TIME           (10*1000UL)
#define   RESTART_RETRY_INTERVAL_TIME           (40*1000UL)
#define   CPIN_RETRY_INTERVAL_TIME              (20*1000UL)
#define   CONNECT_RETRY_INTERVAL_TIME           (20*1000UL)
#define   SEND_RETRY_INTERVAL_TIME              (30*1000UL)
#define   STOP_TASK_INTERVAL_TIME               (10*1000UL)
#define   LOOP_INTERVAL_TIME                    (30*1000UL)
#define   GETINFO_INTERVAL_TIME                 (20*1000UL)

//ص
void Power_ON_t_CallBack(MultiTimer* timer, void* userData)
{
    IotFlag_t.PowerOn_error_flag = 1;
}

//ʱص
void Restart_CallBack(MultiTimer* timer, void* userData)
{
    IotFlag_t.restart_error_flag = 1;
}

//Cpinʱص
void Cpin_CallBack(MultiTimer* timer, void* userData)
{
    IotFlag_t.sim_error_flag = 1;
}

//Connectʱص
void Connect_CallBack(MultiTimer* timer, void* userData)
{
    IotFlag_t.Connect_error_flag = 1; 
}

//GetInfoʱص
void GetInfo_CallBack(MultiTimer* timer, void* userData)
{
    IotFlag_t.ModInfo_error_flag = 1;
}

//Sendʱص
void Send_CallBack(MultiTimer* timer, void* userData)
{
    IotFlag_t.send_error_flag = 1; 
}

// //Loopص
// void Loop_CallBack(MultiTimer* timer, void* userData)
// {
//     state = 0;
//     IotFlag_t.Loop_count_flag++;
//     IotFlag_t.module_init_flag = 1;
//     IotFlag_t.Iot_Retry_flag = 0;
// }

//StopTaskʱص
void StopTask_CallBack(MultiTimer* timer, void* userData)
{
    logDebug("BSP_Ml307_Power_Off\r\n");
    // IotFlag_t.Loop_count_flag = 0;
    // if(IotFlag_t.PowerOn_error_flag == 1)
    // {
        IotFlag_t.PowerOn_error_flag = 0;
    //     tmos_stop_task(ml307_task_id, ML307_PERIODIC_EVT); 
    //     PFIC_DisableIRQ(UART1_IRQn);
    //     BSP_RequestSleep();
    // }else{
        // BSP_Ml307_Power_Off();          //ػ
        tmos_stop_task(ml307_task_id, ML307_PERIODIC_EVT); 
        PFIC_DisableIRQ(UART1_IRQn);
        BSP_RequestSleep();
    // }
    IotFlag_t.Date_Sending_flag = 0;
}


/**
 * @brief ·JSON
 * @param json_str JSONַ
 * @return 0: ɹ, -1: ʧ
 */
static int parse_server_json_data(const char* json_str)
{
    lwjson_t lwjson;
    lwjson_token_t tokens[64];  // tokenԴӵJSONṹ
    const lwjson_token_t* token;
    
    // ʼlwjson
    if (lwjson_init(&lwjson, tokens, LWJSON_ARRAYSIZE(tokens)) != lwjsonOK) {
        logDebug("lwjson init failed\r\n");
        return -1;
    }
    
    // JSONַ
    if (lwjson_parse(&lwjson, json_str) != lwjsonOK) {
        logDebug("JSON parse failed\r\n");
        lwjson_free(&lwjson);
        return -1;
    }
    
    logDebug("JSON parse success\r\n");
    
    // 豸к
    token = lwjson_find(&lwjson, "sn");
    if (token != NULL && token->type == LWJSON_TYPE_STRING) {
        char sn_str[32] = {0};
        size_t len = token->u.str.token_value_len;
        if (len < sizeof(sn_str)) {
            strncpy(sn_str, token->u.str.token_value, len);
            logDebug("Device SN: %s\r\n", sn_str);
        }
    }
    
    // ʱ
    token = lwjson_find(&lwjson, "t");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        uint32_t timestamp = (uint32_t)token->u.num_int;
        logDebug("Timestamp: %u\r\n", timestamp);
    }
    
    // 
    token = lwjson_find(&lwjson, "type");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int msg_type = (int)token->u.num_int;
        logDebug("Message type: %d\r\n", msg_type);
    }
    
    // dataеĸֶ
    
    // ״̬
    token = lwjson_find(&lwjson, "data.switch_status");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int switch_status = (int)token->u.num_int;
        logDebug("Switch status: %d\r\n", switch_status);
        
        // ݿ״ִ̬Ӧ
        if (switch_status == 1) {
            logDebug("Command: Open valve\r\n");
            // BSP_VALVE_Open(); // ȡעִʵʲ
            IotFlag_t.Valve_Open_flag = 1;
        } else if (switch_status == 0) {
            logDebug("Command: Close valve\r\n");
            // BSP_VALVE_Close(); // ȡעִʵʲ
            IotFlag_t.Valve_Close_flag = 1;
        }
    }
    
    // ¶
    token = lwjson_find(&lwjson, "data.temp");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int temperature = (int)token->u.num_int;
        logDebug("Temperature: %dC\r\n", temperature);
    }
    
    // ѹ
    token = lwjson_find(&lwjson, "data.in_pressure");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int in_pressure = (int)token->u.num_int;
        logDebug("In pressure: %d Pa\r\n", in_pressure);
    }
    
    // ѹ
    token = lwjson_find(&lwjson, "data.out_pressure");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int out_pressure = (int)token->u.num_int;
        logDebug("Out pressure: %d Pa\r\n", out_pressure);
    }
    
    // ѹ
    token = lwjson_find(&lwjson, "data.atm_pressure");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int atm_pressure = (int)token->u.num_int;
        logDebug("Atm pressure: %d Pa\r\n", atm_pressure);
    }
    
    // ص
    token = lwjson_find(&lwjson, "data.bat");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int battery = (int)token->u.num_int;
        logDebug("Battery: %d%%\r\n", battery);
    }
    
    // ʪ
    token = lwjson_find(&lwjson, "data.humi");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int humidity = (int)token->u.num_int;
        logDebug("Humidity: %d%%\r\n", humidity);
    }
    
    // RSSI
    token = lwjson_find(&lwjson, "data.rssi");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int rssi = (int)token->u.num_int;
        logDebug("RSSI: %d\r\n", rssi);
    }
    
    // źǿ
    token = lwjson_find(&lwjson, "data.signal");
    if (token != NULL && token->type == LWJSON_TYPE_NUM_INT) {
        int signal = (int)token->u.num_int;
        logDebug("Signal: %d\r\n", signal);
    }
    
    // ͷԴ
    lwjson_free(&lwjson);
    return 0;
}

//ģ׼
static int URC_Module_Ready_Cb(at_urc_info_t *info)
{
    logDebug("module CPIN READY ! \r\n");

    return 0;
}

// MQTTϢջص
static int URC_MQTT_Publish_Cb(at_urc_info_t *info)
{
    // logDebug("MQTT rece: %s\r\n", info->urcbuf);
    
    char topic[64] = {0};
    char payload[512] = {0};

    if (sscanf(info->urcbuf, "+MQTTURC: \"publish\",%*d,%*d,\"%[^\"]\",", topic) >= 1) {
        logDebug("topic: %s\r\n", topic);
        
        // JSONݵʼλãҵһ'{'ַ
        char *json_start = strchr(info->urcbuf, '{');
        if (json_start) {
            logDebug("payload: %s\r\n", json_start);
            
            // JSON
            if (parse_server_json_data(json_start) == 0) {
                logDebug("Server JSON data parsed successfully\r\n");
            } else {
                logDebug("Failed to parse server JSON data\r\n");
            }
            
        }
    }

    return 0;
}

/**
 * @brief urcı
 */
static const urc_item_t urc_table[] =
{
    {.prefix = "+CPIN: READY",       .endmark = '\n',  .handler = URC_Module_Ready_Cb},//ģ׼
    {.prefix = "publish",     .endmark = '\n',  .handler = URC_MQTT_Publish_Cb},//MQTTϢ
};


uint64_t imei_dec = 0;//ʮƵIMEI,ʱӵ
TsModule_Info_t Module_Info_t = {0};
//Convert strings to hexadecimal strings and store them in hex_arry
void Module_String2Hex(char *hex_arry, char *str)
{
    for (int i = 0; i < strlen(str); i++)
    {
        sprintf(hex_arry + (i * 2), "%02X", str[i]);
    }
}

// ȫֱڴ洢豸SN
char device_sn[16] = {0};

// ȡ豸SNMACַ
static void BSP_Get_Device_SN(void)
{
    uint8_t MacAddr[6];
    GetMACAddress(MacAddr);
    
    // 6ֽMACַתΪ12λʮַΪ豸SN
    sprintf(device_sn, "%02X%02X%02X%02X%02X%02X", 
            MacAddr[0], MacAddr[1], MacAddr[2], 
            MacAddr[3], MacAddr[4], MacAddr[5]);
    
    logDebug("Device SN (from MAC): %s\r\n", device_sn);
}

static void Module_Get_Imei_Cb(at_response_t *r)
{
    char imei[16] = {0};
    if (r->code == AT_RESP_OK)
    {
        if (sscanf(r->prefix, "+GSN:%s\r\n", imei) == 1)
        {
            imei_dec = strtoll(imei, NULL, 10);
            strcpy(Module_Info_t.imei, imei);
            sprintf(Module_Info_t.imei_len, "%04X", strlen(Module_Info_t.imei));
            logDebug("Module_Info_t.imei: %s\r\n", Module_Info_t.imei);
        }
    }
    else
    {
        logDebug("'CGSN' command response failed!\r\n");
    }
}

/*
 * @brief ȡIMEI
 * @return
 */
static void Module_Get_Imei(void)
{
    at_attr_t attr;
    at_attr_deinit(&attr);
    attr.prefix = "+GSN:";
    attr.cb = Module_Get_Imei_Cb;
    at_send_singlline(ml307r_obj, &attr, "AT+GSN=1");
}

static void Module_Get_Imsi_Cb(at_response_t *r)
{
    char imsi[60] = {0};
    if (r->code == AT_RESP_OK)
    {
        if (sscanf(r->prefix, "%s\r\n", imsi) == 1)
        {
            strcpy(Module_Info_t.imsi, imsi);
            sprintf( Module_Info_t.imsi_len, "%04X", strlen(Module_Info_t.imsi));
            logDebug("Module_Info_t.imsi: %s\r\n", Module_Info_t.imsi);
        }
    }
    else
    {
        logDebug("'AT+CIMI' command response failed!\r\n");
    }
}

/*
 * @brief ȡIMSI
 * @return
 */
static void Module_Get_Imsi(void)
{
    at_attr_t attr;
    at_attr_deinit(&attr);
    attr.prefix = "46";
    attr.suffix = "\r\n";
    attr.cb = Module_Get_Imsi_Cb; 
    at_send_singlline(ml307r_obj, &attr, "AT+CIMI");
}


static void Module_Get_Iccid_Cb(at_response_t *r)
{
    char iccid[30] = {0};

    if (r->code == AT_RESP_OK)
    {
        //+QCCID: 898604D3102280304162
        if (sscanf(r->prefix, "+MCCID: %s", iccid) == 1)
        {
            strcpy(Module_Info_t.iccid, iccid);
            sprintf( Module_Info_t.iccid_len, "%04X", strlen(Module_Info_t.iccid));
            logDebug("Module_Info_t.iccid: %s\r\n", Module_Info_t.iccid);
        }
    }
    else
    {
        logDebug("'AT+QCCID' command response failed!\r\n");
    }
}

/*
 * @brief ȡICCID
 * @return
 */
static void Module_Get_Iccid(void)
{
    at_attr_t attr;
    at_attr_deinit(&attr);
    attr.prefix = "+MCCID:";
    attr.cb = Module_Get_Iccid_Cb;
    at_send_singlline(ml307r_obj, &attr, "AT+MCCID");
}


//ַȡضʽʱϢ
void find_and_extract(char* string_buf, char* result)
{
    const char *start = "+CCLK: \"";
    const char *end = "+32\"";

    char *s = strstr(string_buf, start);
    if(s)
    {
        s += strlen(start);
        char *e = strstr(s, end);
        if(e)
        {
            *e = '\0';
            strncpy(result, s, e-s+1);
        }
    }
}

/*
 * ʱãĬ϶
 */
#define TIMER_ZONE                              (8U)

// ʱתΪӦʱʱ
static void Time_Zone_Conversion(struct tm* timeInfo)
{
    // ʱתʱΪ8СʱûзӲ졣
    timeInfo -> tm_hour += TIMER_ZONE; //Сʱּ8Сʱ
    if (timeInfo -> tm_hour >= 24)
    {                                  //24Сʱ1Сʱ24
        timeInfo -> tm_mday++;
        timeInfo -> tm_hour -= 24;
        if (timeInfo -> tm_mday >365)
        {
            timeInfo -> tm_year += 1;
        }
    }
}


struct tm Network_Time_t;//ʱ

static void Module_Timer_Cb(at_response_t *r)
{
    int year, mounth, days, hous, min, sec;

    char urc_buf[32] = {0};

    if (r->code == AT_RESP_OK)
    {
        // +CCLK: "24/02/01,05:11:35+32"
        find_and_extract(r->recvbuf, urc_buf);
        if(sscanf(urc_buf, "%d/%d/%d,%d:%d:%d", &year, &mounth, &days, &hous, &min, &sec) == 6)
        {
            logDebug("urc_buf:%s\r\n", urc_buf);
            Network_Time_t.tm_year = (2000 + year);
            Network_Time_t.tm_mon  = mounth;
            Network_Time_t.tm_mday = days;
            Network_Time_t.tm_hour = hous;//ʱΪʱ䣬ҪתΪ
            Network_Time_t.tm_min  = min;
            Network_Time_t.tm_sec  = sec;
            Time_Zone_Conversion(&Network_Time_t);//ʱ

            // RTC_InitTime(Network_Time_t.tm_year, Network_Time_t.tm_mon, Network_Time_t.tm_mday,
            //                               Network_Time_t.tm_hour, Network_Time_t.tm_min, Network_Time_t.tm_sec); //µǰʱ
            // IotFlag_t.update_time_flag = 1;
            IotFlag_t.ModInfo_success_flag = 1;
            logDebug("NET Timer :%04d-%02d-%02d, %02d:%02d:%02d\r\n", Network_Time_t.tm_year, Network_Time_t.tm_mon, Network_Time_t.tm_mday,
                    Network_Time_t.tm_hour, Network_Time_t.tm_min, Network_Time_t.tm_sec);
        }
        else
        {
               logDebug("Network time resolution failed\r\n");//ʱʧ
        }
    }
    else
    {
        logDebug("Network time acquisition failed\r\n");//ʱȡʧ
    }
}

static void Module_Timer(void)
{
    at_attr_t attr;

    at_attr_deinit(&attr);
    attr.cb = Module_Timer_Cb;
    at_send_singlline(ml307r_obj, &attr, "AT+CCLK?");
}

// ʱת
static const uint16_t month_days[] = {0, 31, 28, 31, 30, 31, 30,
                                      31, 31, 30, 31, 30, 31}; // ÿµ

// жǷΪ
static int is_leap_year(int year)
{
    return (year % 4 == 0 && year % 100 != 0) || (year % 400 == 0);
}

static uint32_t utc_to_timestamp(const struct tm *timeinfo)
{
    int year  = timeinfo->tm_year;
    int month = timeinfo->tm_mon; // ·ݴ 1 ʼ
    int day   = timeinfo->tm_mday;
    int hour  = timeinfo->tm_hour;
    int min   = timeinfo->tm_min;
    int sec   = timeinfo->tm_sec;

    int total_days = 0;

    // ۼ
    for (int y = 1970; y < year; y++)
    {
        total_days += 365;
        if (is_leap_year(y))
        {
            total_days += 1;
        }
    }

    // ۼӵĸ·
    for (int m = 1; m < month; m++)
    {
        total_days += month_days[m];
        // ·ݳ2£һ
        if (m == 2 && is_leap_year(year))
        {
            total_days += 1;
        }
    }

    // ϵ
    total_days += day - 1;

    // 
    uint32_t timestamp = ((uint32_t)total_days * 24 * 60 * 60) +
                         ((uint32_t)hour * 60 * 60) +
                         ((uint32_t)min * 60) +
                         sec;

    return timestamp;
}

Ts_Network_Service_Info_t Service_Info_t = {0};

static void Module_NetWork_Info_Cb(at_response_t *r)
{
    if (r->code == AT_RESP_OK)
    {
        //"radio",<rat>,<rsrp>,<rssi>,<tx_power>,<tx_time>,<rx_time>,<last_cellid>,<last_ecl>,<last_sinr>,<last_earfcn>,<last_pci>,<rsrq>
        //+MUESTATS: "radio",4,-660,-400,-32768,0,0,0FB5E606,255,280,38950,22,-60
        char *token = strtok(r->prefix, ",");
        int i = 0;
        while (token != NULL)
        {
            if (i == 0)
            {
                i++;
                token = strtok(NULL, ",");
                continue;
            }
            char hex_str[20] = {0};
            if ((i == 2)||(i == 3)||(i == 4)||(i == 12))
            {
                long int num = strtol((token+1), NULL, 10);
                sprintf(hex_str, "%08X", num);
            }
            else
            {
                if (i != 7)
                {
                    long int num = strtol(token, NULL, 10);
                    if ((i == 2)||(i == 9))
                    {
                        // num = num*10;
                        sprintf(hex_str, "%08X", num);
                    }
                    else
                    {
                        sprintf(hex_str, "%08X", num);
                    }
                }
            }
            switch (i)
            {
                case 1:
                    strcpy(Service_Info_t.rat, hex_str);
                    break;
                case 2:
                    strcpy(Service_Info_t.rsrp, hex_str);
                    break;
                case 3:
                    strcpy(Service_Info_t.rssi, hex_str);
                    break;
                case 4:
                    sprintf(Service_Info_t.tx_power,"%08s", hex_str);

                    break;
                case 5:
                    strcpy(Service_Info_t.tx_time, hex_str);
                    break;
                case 6:
                    strcpy(Service_Info_t.rx_time, hex_str);//rsrpʵֵx10
                    break;
                case 7:
                    strncpy(hex_str, token+1, 8);
                    sprintf(Service_Info_t.last_cellid,"%08s", hex_str);
                    break;
                case 8:
                    strcpy(Service_Info_t.last_ecl, hex_str);//sinrʵֵx10ϱ
                    break;
                case 9:
                    strcpy(Service_Info_t.last_sinr, hex_str);
                    break;
                case 10:
                    strcpy(Service_Info_t.last_earfcn, hex_str);
                    break;
                case 11:
                    strcpy(Service_Info_t.last_pci, hex_str);
                    break;
                case 12:
                    strcpy(Service_Info_t.rsrq, hex_str);
                    break;
                default:
                    break;
            }
            i++;
            token = strtok(NULL, ",");
        }
        //    logDebug("Service_Info_t = %s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\r\n", Service_Info_t.rat,
        //            Service_Info_t.rsrp,Service_Info_t.rssi,Service_Info_t.tx_power,Service_Info_t.tx_time,
        //            Service_Info_t.rx_time,Service_Info_t.last_cellid,Service_Info_t.last_ecl,Service_Info_t.last_sinr,
        //            Service_Info_t.last_earfcn,Service_Info_t.last_pci,Service_Info_t.rsrq);
        IotFlag_t.ModInfo_success_flag = 1;
    }
    else
    {
        logDebug("AT+MUESTATS=radio command response failed!\r\n");
    }
}

/*
 * @brief ȡĲϢ
 * @return
 */
static void Module_NetWork_Info(void)
{
    at_attr_t attr;

    at_attr_deinit(&attr);
    attr.prefix = "+MUESTATS: ";
    attr.cb = Module_NetWork_Info_Cb;
    at_send_singlline(ml307r_obj, &attr, "AT+MUESTATS=\"radio\"");
}


static void BSP_Get_Module_Info(void)
{
    BSP_Get_Device_SN();         // ȡ豸SN
    Module_Get_Imei();
    Module_Get_Imsi();
    Module_Get_Iccid();
    DelayMs(2000);
    Module_Timer();
    // Module_NetWork_Info();   // ȡϢ
}



static int MQTT_CfgConnSub(at_env_t *e)
{
    char data[128] = {0};

    switch (e->state)
    {
        case 0:
            logDebug("MQTT config start!\r\n");
            e->println(e, "AT+MQTTCFG=\"pingresp\",0,1");
            e->reset_timer(e);
            e->state++;
            break;
        case 1:
            if (e->contains(e, "OK"))
            {
                tmos_memset(data, 0, sizeof(data));
                // ʹ豸SNΪͻIDһ
                sprintf(data, "AT+MQTTCONN=0,\"%s\",%d,\"%s-%s\",\"%s\",\"%s\"", 
                        MQTT_HOST, MQTT_PORT, CLIENT_ID, device_sn, MQTT_USER, MQTT_PASS);
                e->println(e, data);
                e->reset_timer(e);
                e->state++;
            }
            if (e->is_timeout(e, 5000))
            {
                e->state--;
                if (++e->i > 3)
                {
                    logDebug("MQTT config error!\r\n");
                    e->finish(e, AT_RESP_ERROR);
                }
            }
            break;
        case 2:
            if (e->contains(e, "\"conn\",0,0"))
            {
                logDebug("MQTTCONN OK\r\n");
                tmos_memset(data, 0, sizeof(data));
                // 
                sprintf(data, "AT+MQTTSUB=0,\"%s%s\",0", MQTT_TOPIC_REV, device_sn);
                logDebug("MQTTSUB: %s\r\n", data);
                e->println(e, data);
                e->reset_timer(e);
                e->state++;
            }
            if (e->is_timeout(e, 3000))
            {
                e->state--;
                if (++e->i > 3)
                {
                    IotFlag_t.Connect_error_flag = 1;
                    logDebug("MQTT connect error!\r\n");
                    e->finish(e, AT_RESP_ERROR);
                }
            }
            break;
        case 3:
            if (e->contains(e, "suback"))
            {
                logDebug("MQTTSUB OK\r\n");
                IotFlag_t.Connect_success_flag = 1;
                e->finish(e, AT_RESP_OK);
            }
            if (e->is_timeout(e, 5000))
            {
                e->state--;
                if (++e->i > 3)
                {
                    logDebug("MQTT subscribe error!\r\n");
                    IotFlag_t.Connect_error_flag = 1;
                    e->finish(e, AT_RESP_ERROR);
                }
            }
            break;
    }
    return 0;
}

static void BSP_MQTT_CfgConnSub(void)
{
    at_do_work(ml307r_obj, NULL, MQTT_CfgConnSub);
}



/*
 * @brief 豸
 * @return 
 */
static int BSP_Module_Connect_CtWing_Handle(at_env_t *e)
{
    switch (e->state)
    {
        case 0:
            logDebug("create device...\r\n");  
            e->println(e, "AT+MIPOPEN=0,\"TCP\",\"112.125.89.8\",46696,,1");  // 8.135.10.183,32994
            e->reset_timer(e);
            e->state++;
            break;
        case 1:
            if (e->contains(e, "CONNECT"))
            {
                IotFlag_t.Connect_success_flag = 1; 
                logDebug("create device complete\r\n");
                e->finish(e, AT_RESP_OK);
            }
            else if (e->is_timeout(e, 5000))
            {
                e->state--;
                if (++e->i > 3)
                {
                    logDebug("create device error\r\n");
                    IotFlag_t.Connect_error_flag = 1; 
                    e->finish(e, AT_RESP_ERROR);
                }
            }
            break;
        default:
            break;
    }
    return 0;
}
//ӷ
static void BSP_Module_Connect_CtWing(void)
{
    BSP_MQTT_CfgConnSub();
    // at_do_work(ml307r_obj, NULL, BSP_Module_Connect_CtWing_Handle);//豸
}


//
void BSP_Module_Send_Data(uint8_t* data, uint8_t len)
{
    at_attr_t attr;
    at_send_data(ml307r_obj, &attr, data, len);
}

// JSONС
#define JSON_BUFFER_SIZE 512

// 4Gϱݽṹ
typedef struct {
    char json_buffer[JSON_BUFFER_SIZE];  // JSONַ
    uint16_t length;                     // JSONַ
} Ts4GJsonData;

/**
 * @brief 4GģϱJSON
 * 
 * @param p4GData JSONݽṹָ
 * @param cmd ͣͱһ
 * @return uint8_t 0ʾɹ
 */
uint8_t BSP_4G_Generate_JsonData(Ts4GJsonData *p4GData, TeFrameCmd cmd)
{
    // JSON
    memset(p4GData->json_buffer, 0, JSON_BUFFER_SIZE);
    
    // ȡ
    TsValveData ValveData;

    // ䷧ݣϱͬ
    ValveData.switch_status = gValveData.switch_status;
    ValveData.temp = ValveRawData.atom_temp;
    ValveData.in_pressure = ValveRawData.in_press_raw;
    ValveData.out_pressure = ValveRawData.out_press_raw;
    ValveData.atm_pressure = ValveRawData.atom_press;
    ValveData.type = kZbf;
    ValveData.bat = BSP_ReadVbat();
    ValveData.humi = 50;
    ValveData.rssi = gValveData.rssi;

    // źǿȣ򵥴Ϊ0-5ֵ
    int signal_strength = 0;
    long rssi = strtol(Service_Info_t.rssi, NULL, 16);
    if (rssi > -90) signal_strength = 5;
    else if (rssi > -100) signal_strength = 4;
    else if (rssi > -110) signal_strength = 3;
    else if (rssi > -120) signal_strength = 2;
    else if (rssi > -130) signal_strength = 1;

    // ȡǰʱ
    uint32_t timestamp = 0;
    if (Network_Time_t.tm_year > 0) {
        struct tm timeinfo;
        timeinfo.tm_year = Network_Time_t.tm_year;
        timeinfo.tm_mon = Network_Time_t.tm_mon;
        timeinfo.tm_mday = Network_Time_t.tm_mday;
        timeinfo.tm_hour = Network_Time_t.tm_hour;
        timeinfo.tm_min = Network_Time_t.tm_min;
        timeinfo.tm_sec = Network_Time_t.tm_sec;
        timestamp = utc_to_timestamp(&timeinfo);
    }

    // MQTTʽJSONַ
    p4GData->length = snprintf(p4GData->json_buffer, JSON_BUFFER_SIZE,
        "{"
        "\"sn\":\"%s\","
        "\"t\":%u,"
        "\"type\":%d,"
        "\"data\":{"
        "\"switch_status\":%d,"
        "\"temp\":%d,"
        "\"in_pressure\":%d,"
        "\"out_pressure\":%d,"
        "\"atm_pressure\":%d,"
        "\"type\":%d,"
        "\"bat\":%d,"
        "\"humi\":%d,"
        "\"rssi\":%d,"
        "\"signal\":%d"
        "}"
        "}",
        device_sn,
        timestamp,
        cmd,
        ValveData.switch_status,
        ValveData.temp,
        ValveData.in_pressure,
        ValveData.out_pressure,
        ValveData.atm_pressure,
        ValveData.type,
        ValveData.bat,
        ValveData.humi,
        ValveData.rssi,
        signal_strength
    );

    // ӡJSONϢ
    // logDebug("MQTT JSON data (cmd=%d):\r\n%s", cmd, p4GData->json_buffer);

    return 0;
}
/**
 * @brief 4GģӦJSON
 * 
 * @param p4GData JSONݽṹָ
 * @param responseCmd Ӧ
 * @param result Ӧ0ɹ0ʧ
 * @return uint8_t 0ʾɹ
 */
uint8_t BSP_4G_Generate_Response(Ts4GJsonData *p4GData, TeFrameCmd responseCmd, uint8_t result)
{
    p4GData->length = snprintf(p4GData->json_buffer, JSON_BUFFER_SIZE,
        "{"
        "\"cmd\":%d,"
        "\"data\":{"
        "\"response_cmd\":%d,"
        "\"result\":%d"
        "}"
        "}",
        kCmdData, // ʹΪӦ
        responseCmd,
        result
    );

    logDebug("4GӦJSON:\r\n%s", p4GData->json_buffer);
    
    return 0;
}

// ȫJSONݽṹ
Ts4GJsonData g4GJsonData;
char *json_data;

static int SendMessage(at_env_t *e)
{
    switch (e->state)
    {
        case 0:
            logDebug("MQTT SendMessage start!\r\n");
            // ʹMQTTJSON
            e->println(e, "AT+MQTTPUB=0,\"%s\",0,0,0,0,\"%s\"", MQTT_TOPIC_UP, json_data);
            e->reset_timer(e);
            e->state++;
            break;
        case 1:
            if (e->contains(e, "OK"))
            {
                logDebug("MQTT SendMessage end!\r\n");
                IotFlag_t.send_complete_flag = 1;
                // ɺͷjson_dataڴ
                tmos_msg_deallocate((uint8_t *)json_data);
                json_data = NULL;
                e->finish(e, AT_RESP_OK);
            }
            if (e->is_timeout(e, 5000))
            {
                e->state--;
                if (++e->i > 3)
                {
                    logDebug("MQTT SendMessage error!\r\n");
                    IotFlag_t.send_complete_flag = 1;
                    // ʧҲҪͷjson_dataڴ
                    tmos_msg_deallocate((uint8_t *)json_data);
                    json_data = NULL;
                    e->finish(e, AT_RESP_ERROR);
                }
            }
            break;
    }
    return 0;
}

static void BSP_SendMessage(void)
{
    at_do_work(ml307r_obj, NULL, SendMessage);
}


void BSP_ML307_SendMessage(void)
{
    // Ĭ
    TeFrameCmd cmd = kCmdData;
    
    // Ӷжȡһ
    TsIotSendRequest request;
    if (lwrb_get_full(&iot_send_queue) >= sizeof(request)) {
        lwrb_read(&iot_send_queue, &request, sizeof(request));
        cmd = request.cmd;
        logDebug("Get command type: %d\r\n", cmd);
    } else {
        // ΪգʹĬ߼
        if (gValveData.switch_status == kClosed)
        {
            cmd = gValveCloseReason;
        }else
        {
            cmd = gValveOpenReason;
        }
    }

    // JSON
    BSP_4G_Generate_JsonData(&g4GJsonData, cmd);
    
    // ڴ沢JSONַ
    json_data = (char *)tmos_msg_allocate(g4GJsonData.length + 1);
    
    if (json_data != NULL) {
        strcpy(json_data, g4GJsonData.json_buffer);
        logDebug("json_data len %d\r\n", strlen(json_data));
        BSP_SendMessage();
    } else {
        logDebug("JSON Memory failure!\n");
        IotFlag_t.send_error_flag = 1;
    }
}

static int Ml307_Power_On(at_env_t *e)
{
    switch (e->state)
    {
        case 0:
            logDebug("Ml307_Power_On start!\r\n");
            GPIOB_SetBits(ML307_PWR_PIN);
            e->reset_timer(e);
            e->state++;
            break;
        case 1:
            #define ML307_POWER_ON_TIME 3
            if (e->is_timeout(e, ML307_POWER_ON_TIME *1000))
            {
                logDebug("Ml307_Power_On end!\r\n");
                GPIOB_ResetBits(ML307_PWR_PIN);
                IotFlag_t.PowerOn_success_flag = 1;
                e->finish(e, AT_RESP_OK);
            }
            break;
    }
    return 0;
}

static void BSP_Ml307_Power_On(void)
{
    at_do_work(ml307r_obj, NULL, Ml307_Power_On);
}


static int Ml307_Power_Off(at_env_t *e)
{
    switch (e->state)
    {
        case 0:
            logDebug("Ml307_Power_Off start!\r\n");
            GPIOB_SetBits(ML307_PWR_PIN);
            e->reset_timer(e);
            e->state++;
            break;
        case 1:
            #define M307_POWER_OFF_TIME 4
            if (e->is_timeout(e, M307_POWER_OFF_TIME * 1000))
            {
                GPIOB_ResetBits(ML307_PWR_PIN);
                logDebug("Ml307_Power_Off end!\r\n");
                e->reset_timer(e);
                e->state++;
           }
            break;
        case 2:
            if (e->is_timeout(e, 1000))
            {
                e->finish(e, AT_RESP_OK);
            }
            break;
    }
    return 0;
}

static void BSP_Ml307_Power_Off(void)
{
    at_do_work(ml307r_obj, NULL, Ml307_Power_Off);
}


static int Module_Restart_Work(at_env_t *e)
{
    switch (e->state)
    {
        case 0:
            logDebug("restart module...\r\n");
            e->println(e, "AT+CFUN=1,1");//ģ
            e->reset_timer(e);
            e->state++;
            break;
        case 1:
            if (e->contains(e, "OK"))
            {
                 logDebug("restart module complete\r\n");
                 e->reset_timer(e);
                 e->state++;
            }
            if (e->is_timeout(e, 1500))
            {
                e->state--;
                if (++e->i > 3)
                {
                    IotFlag_t.restart_error_flag = 1;
                    logDebug("Module restart error\r\n");
                    e->finish(e, AT_RESP_ERROR);
                }
           }
            break;
        case 2:
            logDebug("Check if the Module is ready\r\n");
            e->println(e, "AT");
            e->reset_timer(e);
            e->state++;
            break;
        case 3:
            if (e->contains(e, "OK"))
            {
                IotFlag_t.restart_success_flag = 1;
                e->recvclr(e);
                e->finish(e, AT_RESP_OK);
                logDebug("Module is ready\r\n");
            }
            if (e->is_timeout(e, 5000))
            {
                e->state--;
                if (++e->i > 3)
                {
                    IotFlag_t.restart_error_flag = 1;
                    logDebug("Module  error\r\n");
                    e->finish(e, AT_RESP_ERROR);
                }
            }
            break;
        }
    return 0;
}
//ģ
static void BSP_Restart_Module(void)
{
    at_do_work(ml307r_obj, NULL, Module_Restart_Work);//Ч
}



static int simcom_init(at_env_t *e)
{
    switch (e->state)
    {
        case 0:
            logDebug("simcom_cpin!\r\n");
            e->println(e, "AT+CPIN?");
            e->reset_timer(e);
            e->state++;
            break;
        case 1:
            if (e->contains(e, "+CPIN: READY"))
            {
                IotFlag_t.sim_success_flag = 1;
                logDebug("simcom CPIN READY!\r\n");
                e->finish(e, AT_RESP_OK);
            }
            if (e->is_timeout(e, 3000))
            {
                e->state--;
                if (++e->i > 3)
                {
                    IotFlag_t.sim_error_flag = 1;
                    logDebug("simcom CPIN NO READY!\r\n");
                    e->finish(e, AT_RESP_ERROR);
                }
            }
            break;
    }
    return 0;
}
//CPIN
static void BSP_simcom_init(void)
{
    at_do_work(ml307r_obj, NULL, simcom_init);
}


static void BSP_Iot_Loop(void)
{
    switch (state)
    {
        case IOT_STATE_POWER_ON:                // 
            if(IotFlag_t.module_init_flag)       // ʼ
            {
                logDebug("IotFlag_t.module_init_flag %d\r\n",IotFlag_t.module_init_flag);
                IotFlag_t.module_init_flag = 0;
                MultiTimerStart(&Multi_Power_ON_t,POWERON_RETRY_INTERVAL_TIME, Power_ON_t_CallBack, NULL);
                logDebug("BSP_Ml307_Power_On\r\n");
                BSP_Ml307_Power_On();          
            }
            if (IotFlag_t.PowerOn_success_flag)   // ɹ
            {
                logDebug("BSP_Ml307_Power_On!\r\n");
                MultiTimerStop(&Multi_Power_ON_t);
                IotFlag_t.restart_success_flag = 1;
                state = IOT_STATE_SIM_CHECK;
                IotFlag_t.Iot_Retry_flag = 0;
            }
            if (IotFlag_t.PowerOn_error_flag)     
            {
                logDebug("PowerOn_error_flag!\r\n");
                if (IotFlag_t.Iot_Retry_flag)
                {
                    state = IOT_STATE_PROCESS;
                }
                else
                {
                    IotFlag_t.PowerOn_success_flag = 1;
                    IotFlag_t.Iot_Retry_flag = 1;
                    state = IOT_STATE_RESTART;
                }
            }
            break;
        case IOT_STATE_RESTART:                  // 
            if(IotFlag_t.PowerOn_success_flag)    // 
            {
                IotFlag_t.PowerOn_success_flag = 0;
                MultiTimerStart(&Multi_Restart_t, RESTART_RETRY_INTERVAL_TIME, Restart_CallBack, NULL);
                BSP_Restart_Module();         
            }
            if (IotFlag_t.restart_success_flag)   // ɹ
            {
                logDebug("restart_success_flag!\r\n");
                MultiTimerStop(&Multi_Restart_t);
                state = IOT_STATE_SIM_CHECK;
            }
            if(IotFlag_t.restart_error_flag == 1)  
            {
                MultiTimerStop(&Multi_Restart_t);
                IotFlag_t.restart_error_flag = 0;
                state = IOT_STATE_PROCESS;
            }
            break;
        case IOT_STATE_SIM_CHECK:               // sim״̬
            if(IotFlag_t.restart_success_flag)   // ɹ
            {
                IotFlag_t.restart_success_flag = 0;
                IotFlag_t.PowerOn_success_flag = 0;
                MultiTimerStart(&Multi_Cpin_t, CPIN_RETRY_INTERVAL_TIME, Cpin_CallBack, NULL);
                BSP_simcom_init();               
            }
            if (IotFlag_t.sim_success_flag)       // simɹ
            {
                logDebug("BSP_simcom_init success!!!\r\n");
                MultiTimerStop(&Multi_Cpin_t);
                state = IOT_STATE_GET_MODULE_INFO;
            }
            if(IotFlag_t.sim_error_flag == 1)  
            {
                logDebug("BSP_simcom_init Error !\r\n");
                MultiTimerStop(&Multi_Cpin_t);
                IotFlag_t.sim_error_flag = 0;
                if (IotFlag_t.Iot_Retry_flag)
                {
                    state = IOT_STATE_PROCESS;
                }
                else
                {
                    IotFlag_t.PowerOn_success_flag = 1;
                    IotFlag_t.Iot_Retry_flag = 1;
                    state = IOT_STATE_RESTART;
                }
            }
            break;
        case IOT_STATE_GET_MODULE_INFO:         // ȡģϢ
            if(IotFlag_t.sim_success_flag)       // sim
            {
                IotFlag_t.sim_success_flag = 0;
                MultiTimerStart(&Multi_GetInfo_t, GETINFO_INTERVAL_TIME, GetInfo_CallBack, NULL);
                BSP_Get_Module_Info();           
            }
            if (IotFlag_t.ModInfo_success_flag)   // get success
            {
                logDebug("ModInfo_success_flag!\r\n");
                MultiTimerStop(&Multi_GetInfo_t);
                state = IOT_STATE_CONNECT;
            }
            if(IotFlag_t.ModInfo_error_flag == 1)  
            {
                logDebug("ModInfo_error_flag!\r\n");
                MultiTimerStop(&Multi_GetInfo_t);
                IotFlag_t.ModInfo_error_flag = 0;
                if (IotFlag_t.Iot_Retry_flag)
                {
                    state = IOT_STATE_PROCESS;
                }
                else
                {
                    IotFlag_t.PowerOn_success_flag = 1;
                    IotFlag_t.Iot_Retry_flag = 1;
                    state = IOT_STATE_RESTART;
                }
            }
            break;
        case IOT_STATE_CONNECT:                 // 
            if(IotFlag_t.ModInfo_success_flag)   // ȡϢɹ
            {
                IotFlag_t.ModInfo_success_flag = 0;
                MultiTimerStart(&Multi_Connect_t, CONNECT_RETRY_INTERVAL_TIME, Connect_CallBack, NULL);
                BSP_Module_Connect_CtWing();     
            }
            if (IotFlag_t.Connect_success_flag)   // ӳɹ
            {
                logDebug("Connect_success_flag!\r\n");
                MultiTimerStop(&Multi_Connect_t);
                state = IOT_STATE_SEND_DATA;
            }
            if(IotFlag_t.Connect_error_flag == 1)  
            {
                logDebug("Connect_error_flag!\r\n");
                IotFlag_t.Connect_error_flag = 0;
                MultiTimerStop(&Multi_Connect_t);
                if (IotFlag_t.Iot_Retry_flag)
                {
                    state = IOT_STATE_PROCESS;
                }
                else
                {
                    IotFlag_t.PowerOn_success_flag = 1;
                    IotFlag_t.Iot_Retry_flag = 1;
                    state = IOT_STATE_RESTART;
                }
            }
            break;
        case IOT_STATE_SEND_DATA:              // 
            if(IotFlag_t.Connect_success_flag)   // 
            {
                IotFlag_t.Connect_success_flag = 0;
                MultiTimerStart(&Multi_Send_t, SEND_RETRY_INTERVAL_TIME, Send_CallBack, NULL);
                BSP_ML307_SendMessage();           
            }
            if (IotFlag_t.send_complete_flag)      // ͳɹ
            {
                logDebug("send_complete_flag!\r\n");
                MultiTimerStop(&Multi_Send_t);
                state = IOT_STATE_PROCESS;
            }
            if(IotFlag_t.send_error_flag == 1)  
            {
                logDebug("send_error_flag!\r\n");
                MultiTimerStop(&Multi_Send_t);
                IotFlag_t.send_error_flag = 0;
                if (IotFlag_t.Iot_Retry_flag)
                {
                    state = IOT_STATE_PROCESS;
                }
                else
                {
                    IotFlag_t.PowerOn_success_flag = 1;
                    IotFlag_t.Iot_Retry_flag = 1;
                    state = IOT_STATE_RESTART;
                }
            }
            break;
        case IOT_STATE_PROCESS:                // 
            if(IotFlag_t.send_complete_flag == 1)
            {
                logDebug("send_complete_into_stop\r\n");
                IotFlag_t.send_complete_flag = 0;
                
                // ָǷΪNULLͷţظͷ
                if(json_data != NULL) {
                    tmos_msg_deallocate((uint8_t *)json_data);
                    json_data = NULL;
                }
                
                // Ƿд͵
                if (lwrb_get_full(&iot_send_queue) >= sizeof(TsIotSendRequest)) {
                    logDebug("The queue has data to be sent, continue\r\n");
                    IotFlag_t.Connect_success_flag = 1;  
                    state = IOT_STATE_SEND_DATA;  // ֱת״̬
                } else {
                    logDebug("send finish\r\n");
                    state = IOT_STATE_POWER_ON;   
                    MultiTimerStart(&Multi_StopTask_t, STOP_TASK_INTERVAL_TIME, StopTask_CallBack, NULL);  // ر
                }
            }else
            {
                state = IOT_STATE_POWER_ON;
                IotFlag_t.Iot_Retry_flag = 0; // Ա־λ
                IotFlag_t.module_init_flag = 0;
                MultiTimerStart(&Multi_StopTask_t, STOP_TASK_INTERVAL_TIME, StopTask_CallBack, NULL);
            }
        break;
    }
}

void Ml307r_Loop(void)
{
    FEED_IWDG();
    at_obj_process(ml307r_obj);
    BSP_Iot_Loop();
}

__HIGH_CODE
__attribute__((noinline))
uint16_t Ml307r_Handle(uint8_t task_id, uint16_t events)
{
    if (events & ML307_PERIODIC_EVT)
    {
        Ml307r_Loop();
        tmos_start_task(ml307_task_id, ML307_PERIODIC_EVT, MS1_TO_SYSTEM_TIME(5));  //5
        return (events ^ ML307_PERIODIC_EVT);
    }
    return 0;
}

void BSP_Ml307r_Init(void)
{
    if(IotFlag_t.Date_Sending_flag == 0)
    {
        // ʼ״̬ϱ
        lwrb_init(&iot_send_queue, iot_send_queue_buffer, sizeof(iot_send_queue_buffer));
        BSP_UART1_Init();
        Ml307r_Gpio_Init();
        ml307r_obj = at_obj_create(&at_adapter);//Create AT communication object
        if(ml307r_obj != NULL) 
        {
            at_obj_set_urc(ml307r_obj, urc_table, sizeof(urc_table) / sizeof(urc_table[0]));
            IotFlag_t.module_init_flag = 1;
            IotFlag_t.Date_Sending_flag = 1;
            BSP_BlockSleep();
            ml307_task_id = TMOS_ProcessEventRegister(Ml307r_Handle);
            // tmos_set_event(ml307_task_id, ML307_PERIODIC_EVT);
            tmos_start_task(ml307_task_id, ML307_PERIODIC_EVT, MS1_TO_SYSTEM_TIME(300)); //ȶģ 
        }
    }else
    {
        logDebug("ERROR ,Iot_Send_Data!!!\r\n");
    }
}

void Iot_Send_Data(TeFrameCmd cmd)
{
    // ṹ
    TsIotSendRequest request;
    request.cmd = cmd;
    
    // ӵ
    if (lwrb_write(&iot_send_queue, &request, sizeof(request)) == sizeof(request)) {
        logDebug("add to the queue\r\n");
        
        // ǰûзڽУ
        if(IotFlag_t.Date_Sending_flag == 0)
        {
            BSP_UART1_Init();
            Ml307r_Gpio_Init();
            logDebug("Ml307r_Init_Finish!!!\r\n");
            if(ml307r_obj != NULL) 
            {
                IotFlag_t.Date_Sending_flag = 1;
                IotFlag_t.module_init_flag = 1;
                BSP_BlockSleep();
                ml307_task_id = TMOS_ProcessEventRegister(Ml307r_Handle);
                tmos_start_task(ml307_task_id, ML307_PERIODIC_EVT, MS1_TO_SYSTEM_TIME(300)); 
            }else
            {
                BSP_Ml307r_Init();
            }
        }
    } else {
        logDebug("The queue is full\r\n");
        lwrb_reset(&iot_send_queue);
    }
}
